const { getBabelCommonOptions } = require('../../util/babel');

const babelOptions = getBabelCommonOptions();

// https://github.com/babel/babel/tree/main/eslint/babel-eslint-parser
module.exports = {
  // ESLint's default parser and core rules only support the latest final ECMAScript standard and do not support experimental (such as new features) and non-standard (such as Flow or TypeScript types) syntax provided by Babel. @babel/eslint-parser is a parser that allows ESLint to run on source code that is transformed by Babel.
  parser: '@babel/eslint-parser',

  // The parserOptions described in the official documentation are for the default parser and are not necessarily supported by @babel/eslint-parser.
  parserOptions: {
    // set "latest" to use the most recently supported ECMAScript version.
    ecmaVersion: 'latest',
    sourceType: 'module',
    // Please note that the ecmaFeatures config property may still be required for ESLint to work properly with features not in ECMAScript 5 by default.
    ecmaFeatures: {
      globalReturn: false,
      jsx: true,
    },
    requireConfigFile: false,
    // an object containing Babel configuration options that are passed to Babel's parser at runtime.
    babelOptions,
  },

  // Companion rules for @babel/eslint-parser. @babel/eslint-parser does a great job at adapting eslint for use with Babel, but it can't change the built-in rules to support experimental features. @babel/eslint-plugin re-implements problematic rules so they do not give false positives or negatives.
  plugins: ['@babel'],
  // Finally enable all the rules you would like to use (remember to disable the original ones as well!).
  rules: {
    // require a capital letter for constructors
    'new-cap': 0,
    '@babel/new-cap': 'warn',

    // disallow this keywords outside of classes or class-like objects
    'no-invalid-this': 0,
    '@babel/no-invalid-this': 0,

    // disallow usage of expressions in statement position
    'no-unused-expressions': 0,
    '@babel/no-unused-expressions': 0,

    // require padding inside curly braces
    // prettier
    'object-curly-spacing': 0,
    '@babel/object-curly-spacing': 0,

    // require or disallow use of semicolons instead of ASI
    // prettier
    semi: 0,
    '@babel/semi': 0,
  },
};
