// copy and modify from ice/packages/build-babel-config/src/index.ts

function resolvePlugin(plugins) {
  return plugins.filter(Boolean).map((plugin) => {
    if (Array.isArray(plugin)) {
      const [pluginName, ...args] = plugin;
      return [require.resolve(pluginName), ...args];
    }
    return require.resolve(plugin);
  });
}

function getBabelOptions(opts = {}) {
  const plugins = [
    // Stage 0
    '@babel/plugin-proposal-function-bind',
    // Stage 1
    '@babel/plugin-proposal-export-default-from',
    '@babel/plugin-proposal-logical-assignment-operators',
    ['@babel/plugin-proposal-optional-chaining', { loose: false }],
    ['@babel/plugin-proposal-pipeline-operator', { proposal: 'minimal' }],
    ['@babel/plugin-proposal-nullish-coalescing-operator', { loose: false }],
    '@babel/plugin-proposal-do-expressions',
    // Stage 2
    ['@babel/plugin-proposal-decorators', { legacy: true }],
    '@babel/plugin-proposal-function-sent',
    '@babel/plugin-proposal-export-namespace-from',
    '@babel/plugin-proposal-numeric-separator',
    '@babel/plugin-proposal-throw-expressions',
    // Stage 3
    '@babel/plugin-syntax-dynamic-import',
    '@babel/plugin-syntax-import-meta',
    // Note:
    // 'loose' mode configuration must be the same for
    // - @babel/plugin-proposal-class-properties
    // - @babel/plugin-proposal-private-methods
    // - @babel/plugin-proposal-private-property-in-object
    ['@babel/plugin-proposal-class-properties', { loose: true }],
    ['@babel/plugin-proposal-private-methods', { loose: true }],
    ['@babel/plugin-proposal-private-property-in-object', { loose: true }],
    '@babel/plugin-proposal-json-strings',
  ];

  return {
    presets: resolvePlugin([
      opts.env && ['@babel/preset-env', opts.env],
      opts.typescript && '@babel/preset-typescript',
      opts.react &&
        (typeof opts.react === 'boolean'
          ? '@babel/preset-react'
          : ['@babel/preset-react', opts.react]),
    ]),
    plugins: resolvePlugin(plugins),
  };
}

function getBabelCommonOptions() {
  return getBabelOptions({
    react: {},
    typescript: true,
    env: {
      modules: false,
      useBuiltIns: 'entry',
      // Set latest polyfills (until ECMAScript 2021). https://unpkg.alibaba-inc.com/browse/core-js-compat@3.18.3/modules-by-versions.json
      corejs: '3.7',
    },
  });
}

module.exports = { getBabelCommonOptions, getBabelOptions };
