function packBaseRestrictedSyntaxArr() {
  return [
    {
      selector: 'ForInStatement',
      message:
        'for..in loops iterate over the entire prototype chain, which is virtually never what you want. Use Object.keys, and iterate over the resulting array.',
    },
    {
      selector: 'LabeledStatement',
      message:
        'Labels are a form of GOTO; using them makes code confusing and hard to maintain and understand.',
    },
    {
      selector: 'WithStatement',
      message:
        '`with` is disallowed in strict mode because it makes code impossible to predict and optimize.',
    },
  ];
}

function packRestrictedSyntaxRules(syntaxArr, kvArr) {
  const rules = {
    // disallow certain syntax forms
    // https://eslint.org/docs/rules/no-restricted-syntax
    // https://eslint.org/docs/developer-guide/selectors#what-syntax-can-selectors-have
    'no-restricted-syntax': ['error', ...syntaxArr],
  };
  if (kvArr) {
    rules['no-restricted-properties'] = ['error', ...kvArr];
  }
  return rules;
}

/**
 * node 禁用的语法
 */
function packNodeRsr() {
  /* we can use generator in node */
  return packRestrictedSyntaxRules(packBaseRestrictedSyntaxArr());
}
exports.packNodeRsr = packNodeRsr;

/**
 * web 禁用的语法
 */
function packAppRsr() {
  /* we can use generator in web */
  return packRestrictedSyntaxRules(packBaseRestrictedSyntaxArr());
}
exports.packAppRsr = packAppRsr;

/**
 * lib 禁用的语法
 */
function packLibRsr() {
  return packRestrictedSyntaxRules(packBaseRestrictedSyntaxArr());
}
exports.packLibRsr = packLibRsr;
