// https://github.com/typescript-eslint/typescript-eslint/blob/main/docs/linting/README.md
// https://github.com/typescript-eslint/typescript-eslint/blob/main/docs/linting/TYPED_LINTING.md
// https://github.com/typescript-eslint/typescript-eslint/blob/main/docs/linting/TROUBLESHOOTING.md
module.exports = {
  // https://eslint.org/docs/user-guide/configuring/configuration-files#how-do-overrides-work
  getTsConfig() {
    return {
      files: ['**/*.{ts,tsx}'],

      // 不能用 require.resolve
      parser: '@typescript-eslint/parser',
      parserOptions: {
        ecmaVersion: 'latest',
        ecmaFeatures: {
          globalReturn: false,
          jsx: true,
        },
        // This option allows you to provide a path to your project's tsconfig.json.
        // This setting is required if you want to use rules which require type information.
        // Relative paths are interpreted relative to the current working directory if tsconfigRootDir is not set.
        project: ['./tsconfig.json', './packages/*/tsconfig.json'],
        extraFileExtensions: ['.vue'],
      },

      // 不能用 require.resolve
      plugins: ['@typescript-eslint'],

      extends: [
        'plugin:@typescript-eslint/recommended',
        'plugin:@typescript-eslint/recommended-requiring-type-checking',
        // 在这里再引入一次 prettier 做覆盖
        'prettier',
      ],

      rules: {
        /**
         * typescript-eslint 的某些规则要先禁用对应的 eslint 的规则
         */
        'no-undef': 0,

        'no-unused-vars': 0,
        '@typescript-eslint/no-unused-vars': 0,
        '@typescript-eslint/adjacent-overload-signatures': 'warn',
        '@typescript-eslint/no-empty-function': 0,
        '@typescript-eslint/no-empty-interface': 0,
        // prettier
        '@typescript-eslint/no-extra-semi': 0,
        '@typescript-eslint/no-inferrable-types': 'warn',
        '@typescript-eslint/no-unnecessary-type-constraint': 'warn',
        '@typescript-eslint/await-thenable': 'warn',
        '@typescript-eslint/no-floating-promises': 'warn',
        '@typescript-eslint/prefer-includes': 0,
        '@typescript-eslint/prefer-string-starts-ends-with': 0,
        '@typescript-eslint/require-await': 0,
        '@typescript-eslint/ban-ts-comment': 0,
        'no-shadow': 0,
        '@typescript-eslint/no-shadow': ['warn'],
        'no-redeclare': 0,
        '@typescript-eslint/no-redeclare': ['error'],

        // 有一些规则 @typescript-eslint/recommended 重新定义了, 在这里覆盖一下
        'no-var': 'warn',
        'prefer-const': 'warn',
        'prefer-rest-params': 'warn',
        'prefer-spread': 'warn',

        // jsx related
        '@typescript-eslint/no-misused-promises': [
          'error',
          {
            checksVoidReturn: false,
          },
        ],
      },
    };
  },
};
